%% add path
addpath('code_LVGP');
addpath('code_AddGP');
addpath('test_functions');
addpath('dataset');

%% Math funciton 1
% load data
load('bending_60_data.mat', 'X_cell', 'y_cell','x_pred', 'y_true');

% X_cell and y are 30 training sets
X = X_cell{1}; y = y_cell{1}; % select the first replicate

d = 3; % number of input variables
d_quan = 2; % number of quantitative variables
d_qual = 1; % number of qualitative variables
dim_qual = [3]; % dimension of qualitative variables, in this example, the 3rd dimension
d_lv = 2; % set the dimension of latent variable to be 2
levels = [6]; % levels of the qualitative variable in this example

n_starts = 200; % number of initial points of hyperparameters

%% fit Latent Variable model
model_LV = gpFit(X,y, n_starts, dim_qual, d_lv, levels);
y_LV = gpPredict(model_LV, x_pred);

%% fit MC model
X_MC = [X(:,1:min(dim_qual)-1),dummyvar(X(:,dim_qual))]; 
x_pred_MC = [x_pred(:,1:min(dim_qual)-1),dummyvar(x_pred(:,dim_qual))];
model_MC = gpFit(X_MC, y, n_starts);
y_MC = gpPredict(model_MC, x_pred_MC);

%% fit UC model
X_UC = toIndicator(X, dim_qual); 
x_pred_UC = toIndicator(x_pred, dim_qual);
model_UC = gpFit(X_UC, y, n_starts);
y_UC = gpPredict(model_UC, x_pred_UC);

%% fit Add_UC model
[modelADD, y_add] = UCADDGP(X, y, d_quan, d_qual, levels, x_pred);

%% compare rrmse
rrmse_LV = sqrt(mean((y_LV-y_true).^2)/var(y_true));
rrmse_MC = sqrt(mean((y_MC-y_true).^2)/var(y_true));
rrmse_UC = sqrt(mean((y_UC-y_true).^2)/var(y_true));
rrmse_add = sqrt(mean((y_add-y_true).^2)/var(y_true));

%% plot 2D latent variables
plotLatent2D(model_LV);